#version 460

//uniform
uniform vec4 diffuse;
uniform float shininess;
uniform vec4 l_dir; // world space
uniform mat4 m_view;

// input
in vec3 n; // normal in camera space
in vec4 p; // postion in camera space

// output
out vec4 color;

void main() {

    // compute direction from the pxel to the eye
    // camera position in camera space is (0,0,0)
    vec3 eye_dir = normalize(vec3(-p)); 

    // compute light direction in camera space
    vec3 l = normalize(vec3(m_view * -l_dir));

    // normalize again to ensure correct magnitude
    vec3 nn = normalize(n);

    // compute diffuse intensity
    float i = max(0.0, dot(l,nn));

    // initial value for specular color
    vec4 specular = vec4(0.0);

    // only compute specular component if light is facing the surface
    if (i > 0.0) {
        // compute half-vector
        vec3 h = normalize(eye_dir + l);
        specular = vec4(1.0) * pow (max(0.0,dot(h,nn)), shininess);
    }

    color = max(0.25, i) * diffuse + specular;
}